#!/usr/bin/env python

# https://github.com/rveachkc/pymsteams/
# reference: https://dev.outlook.com/connectors/reference
import requests


class TeamsWebhookException(Exception):
    """custom exception for failed webhook call"""
    pass


class cardsection:

    def title(self, stitle):
        # title of the section
        self.payload["title"] = stitle
        return self

    def activityTitle(self, sactivityTitle):
        # Title of the event or action. Often this will be the name of the "actor".
        self.payload["activityTitle"] = sactivityTitle
        return self

    def activitySubtitle(self, sactivitySubtitle):
        # A subtitle describing the event or action. Often this will be a summary of the action.
        self.payload["activitySubtitle"] = sactivitySubtitle
        return self

    def activityImage(self, sactivityImage):
        # URL to image or a data URI with the base64-encoded image inline.
        # An image representing the action. Often this is an avatar of the "actor" of the activity.
        self.payload["activityImage"] = sactivityImage
        return self

    def activityText(self, sactivityText):
        # A full description of the action.
        self.payload["activityText"] = sactivityText
        return self

    def addFact(self, factname, factvalue):
        if "facts" not in self.payload.keys():
            self.payload["facts"] = []

        newfact = {
            "name": factname,
            "value": factvalue
        }
        self.payload["facts"].append(newfact)
        return self

    def addImage(self, simage, ititle=None):
        if "images" not in self.payload.keys():
            self.payload["images"] = []
        imobj = {}
        imobj["image"] = simage
        if ititle:
            imobj["title"] = ititle
        self.payload["images"].append(imobj)
        return self

    def text(self, stext):
        self.payload["text"] = stext
        return self

    def linkButton(self, buttontext, buttonurl):
        self.payload["potentialAction"] = [
            {
                "@context": "http://schema.org",
                "@type": "ViewAction",
                "name": buttontext,
                "target": [buttonurl]
            }
        ]
        return self

    def disableMarkdown(self):
        self.payload["markdown"] = False
        return self

    def enableMarkdown(self):
        self.payload["markdown"] = True
        return self

    def dumpSection(self):
        return self.payload

    def __init__(self):
        self.payload = {}


class potentialaction:

    def addInput(self, _type, _id, title, isMultiline=None):
        if "inputs" not in self.payload.keys():
            self.payload["inputs"] = []
        if (self.choices.dumpChoices() == []):
            input = {
                "@type": _type,
                "id": _id,
                "isMultiline": isMultiline,
                "title": title
            }
        else:
            input = {
                "@type": _type,
                "id": _id,
                "isMultiline": str(isMultiline).lower(),
                "choices": self.choices.dumpChoices(),
                "title": title
            }

        self.payload["inputs"].append(input)
        return self

    def addAction(self, _type, _name, _target,_body=None):
        if "actions" not in self.payload.keys():
            self.payload["actions"] = []
        action = {
            "@type": _type,
            "name": _name,
            "target": _target
        }
        if _body:
            action["body"] = _body

        self.payload["actions"].append(action)
        return self

    def addOpenURI(self, _name, _targets):
        """
        Creates a OpenURI action

        https://docs.microsoft.com/en-us/outlook/actionable-messages/message-card-reference#openuri-action

        :param _name: *Name of the text to appear inside the ActionCard*
        :type _name: str
        :param _targets: *A list of dictionaries, ex: `{"os": "default", "uri": "https://www..."}`*
        :type _targets: list(dict())
        """
        self.payload["@type"] = "OpenUri"
        self.payload["name"] = _name
        if not isinstance(_targets, list):
            raise TypeError("Target must be of type list(dict())")
        self.payload["targets"] = _targets
        return self

    def dumpPotentialAction(self):
        return self.payload

    def __init__(self, _name, _type="ActionCard"):
        self.payload = {}
        self.payload["@type"] = _type
        self.payload["name"] = _name
        self.choices = choice()


class choice:
    def __init__(self):
        self.choices = []

    def addChoices(self, _display, _value):
        self.choices.append({
            "display": _display,
            "value": _value
        })

    def dumpChoices(self):
        return self.choices


class connectorcard:

    def text(self, mtext):
        self.payload["text"] = mtext
        return self

    def title(self, mtitle):
        self.payload["title"] = mtitle
        return self

    def summary(self, msummary):
        self.payload["summary"] = msummary
        return self

    def color(self, mcolor):
        if mcolor.lower() == "red":
            self.payload["themeColor"] = "E81123"
        else:
            self.payload["themeColor"] = mcolor
        return self

    def addLinkButton(self, buttontext, buttonurl):
        if "potentialAction" not in self.payload:
            self.payload["potentialAction"] = []

        thisbutton = {
            "@context": "http://schema.org",
            "@type": "ViewAction",
            "name": buttontext,
            "target": [buttonurl]
        }

        self.payload["potentialAction"].append(thisbutton)
        return self

    def newhookurl(self, nhookurl):
        self.hookurl = nhookurl
        return self

    def addSection(self, newsection):
        # this function expects a cardsection object
        if "sections" not in self.payload.keys():
            self.payload["sections"] = []

        self.payload["sections"].append(newsection.dumpSection())
        return self

    def addPotentialAction(self, newaction):
        # this function expects a potential action object
        if "potentialAction" not in self.payload.keys():
            self.payload["potentialAction"] = []

        self.payload["potentialAction"].append(newaction.dumpPotentialAction())
        return self

    def printme(self):
        print("hookurl: %s" % self.hookurl)
        print("payload: %s" % self.payload)

    def send(self):
        headers = {"Content-Type": "application/json"}
        r = requests.post(
            self.hookurl,
            json=self.payload,
            headers=headers,
            proxies=self.proxies,
            timeout=self.http_timeout,
            verify=self.verify,
        )
        self.last_http_response = r

        if r.status_code == requests.codes.ok:  # pylint: disable=no-member
            return True
        else:
            raise TeamsWebhookException(r.text)

    def __init__(self, hookurl, http_proxy=None, https_proxy=None, http_timeout=60, verify=None):
        self.payload = {}
        self.hookurl = hookurl
        self.proxies = {}
        self.http_timeout = http_timeout
        self.verify = verify
        self.last_http_response = None

        if http_proxy:
            self.proxies['http'] = http_proxy

        if https_proxy:
            self.proxies['https'] = https_proxy

        if not self.proxies:
            self.proxies = None


class async_connectorcard(connectorcard):

    async def send(self):
        try:
            import httpx
        except ImportError as e:
            print("For use the asynchronous connector card, "
                  "install the asynchronous version of the library via pip: pip install pymsteams[async]")
            raise e

        headers = {"Content-Type": "application/json"}

        async with httpx.AsyncClient(proxies=self.proxies, verify=self.verify) as client:
            resp = await client.post(self.hookurl,
                                     json=self.payload,
                                     headers=headers,
                                     timeout=self.http_timeout)
            self.last_http_response = resp
            if resp.status_code == httpx.codes.OK and resp.text == '1':
                return True
            else:
                raise TeamsWebhookException(resp.text)


def formaturl(display, url):
    mdurl = "[%s](%s)" % (display, url)
    return mdurl



import asyncio
import os
import sys
import pytest

# add scripts to the path
sys.path.append(
    os.path.split(
        os.path.dirname(
            os.path.abspath(__file__)
        )
    )[0]
)

import pymsteams

def test_env_webhook_url():
    """
        Test that we have the webhook set as an environment variable.
        This is testing our test environment, not the code.
    """
    webhook_url = os.getenv("MS_TEAMS_WEBHOOK", None)
    assert webhook_url
    assert webhook_url.find("https") == 0

def test_send_message():
    """
        This sends a simple text message with a title and link button.
    """

    teams_message = pymsteams.connectorcard(os.getenv("MS_TEAMS_WEBHOOK"))
    teams_message.text("This is a simple text message.")
    teams_message.title("Simple Message Title")
    teams_message.addLinkButton("Go to the Repo", "https://github.com/rveachkc/pymsteams")
    # teams_message.send()

    # assert isinstance(teams_message.last_http_response.status_code, int)


def test_async_send_message():
    """
        This asynchronously send a simple text message with a title and link button.
    """

    loop = asyncio.get_event_loop()

    teams_message = pymsteams.async_connectorcard(os.getenv("MS_TEAMS_WEBHOOK"))
    teams_message.text("This is a simple text message.")
    teams_message.title("Simple Message Title")
    teams_message.addLinkButton("Go to the Repo", "https://github.com/rveachkc/pymsteams")
    # loop.run_until_complete(teams_message.send())

    # assert isinstance(teams_message.last_http_response.status_code, int)


def test_send_sectioned_message():
    """
        This sends a message with sections.
    """

    # start the message
    teams_message = pymsteams.connectorcard(os.getenv("MS_TEAMS_WEBHOOK"))
    teams_message.text("This is the main title.")
    teams_message.title("Sectioned Message Title")

    # section 1
    section_1 = pymsteams.cardsection()
    section_1.title("Section 1 title")
    section_1.activityTitle("my activity title")
    section_1.activitySubtitle("my activity subtitle")
    section_1.activityImage("https://raw.githubusercontent.com/rveachkc/pymsteams/develop/test/desk_toys_1.jpg")
    section_1.activityText("This is my activity Text.  You should see an activity image, activity title, activity subtitle, and this text (of course).")
    section_1.addFact("Fact", "this is fine")
    section_1.addFact("Fact", "this is also fine")
    section_1.text("This is my section 1 text.  This section has an activity above and two facts below.")
    teams_message.addSection(section_1)

    # section 2
    section_2 = pymsteams.cardsection()
    section_2.text("This is section 2.  You should see an image.  This section does not have facts or a title.")
    section_2.addImage("https://raw.githubusercontent.com/rveachkc/pymsteams/develop/test/desk_toys_2.jpg", ititle="Pew Pew Pew")
    teams_message.addSection(section_2)


    # send
    # teams_message.send()
    # assert isinstance(teams_message.last_http_response.status_code, int)


def test_send_potential_action():
    """
        This sends a message with a potential action
    """

    myTeamsMessage = pymsteams.connectorcard(os.getenv("MS_TEAMS_WEBHOOK"))
    myTeamsMessage.text("This message should have four potential actions.")
    myTeamsMessage.title("Action Message Title")

    myTeamsPotentialAction1 = pymsteams.potentialaction(_name = "Add a comment")
    myTeamsPotentialAction1.addInput("TextInput","comment","Add a comment",False)
    myTeamsPotentialAction1.addAction("HttpPost","Add Comment","https://jsonplaceholder.typicode.com/posts")

    myTeamsPotentialAction2 = pymsteams.potentialaction(_name = "Get Users")
    myTeamsPotentialAction2.addInput("DateInput","dueDate","Enter due date")
    myTeamsPotentialAction2.addAction("HttpPost","save","https://jsonplaceholder.typicode.com/posts")

    myTeamsPotentialAction3 = pymsteams.potentialaction(_name = "Change Status")
    myTeamsPotentialAction3.choices.addChoices("In progress","0")
    myTeamsPotentialAction3.choices.addChoices("Active","1")
    myTeamsPotentialAction3.addInput("MultichoiceInput","list","Select a status",False)
    myTeamsPotentialAction3.addAction("HttpPost","Save","https://jsonplaceholder.typicode.com/posts")

    myTeamsPotentialAction4 = pymsteams.potentialaction(_name = "Download pymsteams")
    myTeamsPotentialAction4.addOpenURI("Links", [
        {
            "os": "default",
            "uri": "https://pypi.org/project/pymsteams/",
        },
    ])

    myTeamsMessage.addPotentialAction(myTeamsPotentialAction1)
    myTeamsMessage.addPotentialAction(myTeamsPotentialAction2)
    myTeamsMessage.addPotentialAction(myTeamsPotentialAction3)
    myTeamsMessage.summary("Message Summary")

    # myTeamsMessage.send()
    # assert isinstance(myTeamsMessage.last_http_response.status_code, int)


def test_http_500():
    with pytest.raises(pymsteams.TeamsWebhookException):
        #myTeamsMessage = pymsteams.connectorcard(os.getenv("MS_TEAMS_WEBHOOK"))
        myTeamsMessage = pymsteams.connectorcard("https://httpstat.us/500")
        myTeamsMessage.text("This is a simple text message.")
        myTeamsMessage.title("Simple Message Title")
        myTeamsMessage.send()
        #myTeamsMessage.hookurl = "https://httpstat.us/500"
    

def test_http_403():
    with pytest.raises(pymsteams.TeamsWebhookException):
        myTeamsMessage = pymsteams.connectorcard("http://httpstat.us/403")
        myTeamsMessage.text("This is a simple text message.")
        myTeamsMessage.title("Simple Message Title")
        myTeamsMessage.send()

def test_message_size():
    def getMsg(card):
        msg = pymsteams.connectorcard(os.getenv("MS_TEAMS_WEBHOOK"))
        msg.title('Simple Message Title')
        msg.summary('Simple Summary')
        msg.addSection(card)
        return msg

    # setup text that's too large
    failure_char_count = 21000
    text = 'a' * failure_char_count

    card = pymsteams.cardsection()
    card.text(text)
    msg = getMsg(card)
    with pytest.raises(pymsteams.TeamsWebhookException):
        msg.send()

    card1 = pymsteams.cardsection()
    card2 = pymsteams.cardsection()
    card1.text(text[:int(len(text)/2)])
    card2.text(text[int(len(text)/2):])
    msg = getMsg(card1)
    # assert msg.send()
    msg = getMsg(card2)
    # assert msg.send()


def test_chaining():
    card = pymsteams.cardsection()
    card.title("Foo").activityTitle("Bar").activitySubtitle("Baz")
    assert card.payload["title"] == "Foo"
    assert card.payload["activityTitle"] == "Bar"
    assert card.payload["activitySubtitle"] == "Baz"

    connector_card = pymsteams.connectorcard("https://example.org")
    connector_card.text("Big text").title("Cool title").summary("Something happened")
    assert connector_card.payload["title"] == "Cool title"
    assert connector_card.payload["text"] == "Big text"
    assert connector_card.payload["summary"] == "Something happened"



#!/usr/bin/env python

import os
import sys

from setuptools import setup
from setuptools.command.install import install

VERSION = "0.2.2"


def readme():
    """ print long description """
    with open('README.md') as f:
        long_descrip = f.read()
    return long_descrip


class VerifyVersionCommand(install):
    """Custom command to verify that the git tag matches our version"""
    description = 'verify that the git tag matches our version'

    def run(self):
        tag = os.getenv('CIRCLE_TAG')

        if tag != VERSION:
            info = "Git tag: {0} does not match the version of this app: {1}".format(
                tag, VERSION
            )
            sys.exit(info)


setup(
    name="pymsteams",
    version=VERSION,
    description="Format messages and post to Microsoft Teams.",
    long_description=readme(),
    long_description_content_type="text/markdown",
    url="https://github.com/rveachkc/pymsteams",
    author="Ryan Veach",
    author_email="rveach@gmail.com",
    license="Apache",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Internet",
        "Programming Language :: Python :: 3",
        "Topic :: Office/Business",
        "Topic :: Office/Business :: Groupware",
    ],
    keywords=['Microsoft', 'Teams'],
    packages=['pymsteams'],
    install_requires=[
        'requests>=2.20.0',
    ],
    extras_require={
        "async": ["httpx>=0.18.2"]
    },
    python_requires='>=3.6',
    cmdclass={
        'verify': VerifyVersionCommand,
    }
)
